@@ -5,11 +5,10 @@ import re
 import sys
 import yaml
 import argparse
-from enum import Enum
-from argparse import Namespace, Action, SUPPRESS, _StoreAction, _HelpAction, _SubParsersAction
+from typing import Callable, Tuple, Type, Union
+from argparse import Namespace, Action, SUPPRESS, _HelpAction, _SubParsersAction
 
 from .optionals import get_config_read_mode, FilesCompleterMethod
-from .typing import restricted_number_type, registered_types
 from .util import (
     yamlParserError,
     yamlScannerError,
@@ -19,8 +18,8 @@ from .util import (
     import_object,
     change_to_path_dir,
     Path,
+    _get_key_value,
     _load_config,
-    _flat_namespace_to_dict,
     _dict_to_flat_namespace,
     _issubclass
 )
@@ -29,15 +28,13 @@ from .util import (
 __all__ = [
     'ActionConfigFile',
     'ActionYesNo',
-    'ActionEnum',
-    'ActionOperators',
     'ActionParser',
     'ActionPath',
     'ActionPathList',
 ]
 
 
-def _find_action(parser, dest:str):
+def _find_action(parser, dest:str, within_subcommands:bool=False, exclude=None):
     """Finds an action in a parser given its dest.
 
     Args:
@@ -47,13 +44,48 @@ def _find_action(parser, dest:str):
     Returns:
         Action or None: The action if found, otherwise None.
     """
-    for action in parser._actions:
-        if action.dest == dest \
-           or isinstance(action, _ActionSubCommands) and dest in action._name_parser_map:
+    actions = filter_default_actions(parser._actions)
+    if exclude is not None:
+        actions = [a for a in actions if not isinstance(a, exclude)]
+    for action in actions:
+        if action.dest == dest:
             return action
+        elif isinstance(action, _ActionSubCommands):
+            if dest in action._name_parser_map:
+                return action
+            elif within_subcommands and dest.split('.', 1)[0] in action._name_parser_map:
+                subcommand, subdest = dest.split('.', 1)
+                subparser = action._name_parser_map[subcommand]
+                return _find_action(subparser, subdest, True, exclude=exclude)
     return None
 
 
+def _find_parent_action(parser, key:str, exclude=None):
+    action = _find_action(parser, key, exclude=exclude)
+    if action is None and '.' in key:
+        parts = key.split('.')
+        for n in reversed(range(len(parts)-1)):
+            action = _find_action(parser, '.'.join(parts[:n+1]), exclude=exclude)
+            if action is not None:
+                break
+    return action
+
+
+def _find_parent_actions(parser, key:str, exclude=None):
+    action = _find_parent_action(parser, key, exclude=exclude)
+    if action is None:
+        actions = filter_default_actions(parser._actions)
+        if exclude is not None:
+            actions = [a for a in actions if not isinstance(a, exclude)]
+        parts = key.split('.')
+        for n in reversed(range(len(parts))):
+            prefix = '.'.join(parts[:n+1])+'.'
+            action = [a for a in actions if a.dest.startswith(prefix)]
+            if action != []:
+                break
+    return None if action == [] else action
+
+
 def _is_action_value_list(action:Action):
     """Checks whether an action produces a list value.
 
@@ -80,11 +112,10 @@ def _remove_actions(parser, types):
 
 
 def filter_default_actions(actions):
-    default = (_HelpAction, _ActionPrintConfig)
+    default = (_HelpAction, _ActionHelpClassPath, _ActionPrintConfig)
     if isinstance(actions, list):
         return [a for a in actions if not isinstance(a, default)]
-    else:
-        return {k: a for k, a in actions.items() if not isinstance(a, default)}
+    return {k: a for k, a in actions.items() if not isinstance(a, default)}
 
 
 class ActionConfigFile(Action, FilesCompleterMethod):
@@ -149,26 +180,41 @@ class _ActionPrintConfig(Action):
         kwargs = {'skip_none': False}
         if value is not None and 'skip_null' in value:
             kwargs['skip_none'] = True
-        parser._print_config = kwargs
+        parser.print_config = kwargs
+
+    @staticmethod
+    def print_config_if_requested(parser, cfg):
+        if hasattr(parser, 'print_config') and not hasattr(parser, 'print_config_skip'):
+            sys.stdout.write(parser.dump(cfg, skip_check=True, **parser.print_config))
+            parser.exit()
 
 
 class _ActionConfigLoad(Action):
 
-    def __init__(self, **kwargs):
-        kwargs['help'] = SUPPRESS
-        kwargs['default'] = SUPPRESS
-        super().__init__(**kwargs)
+    def __init__(
+        self,
+        basetype: Type = None,
+        **kwargs
+    ):
+        if len(kwargs) == 0:
+            self._basetype = basetype
+        else:
+            self.basetype = kwargs.pop('_basetype', None)
+            kwargs['help'] = SUPPRESS
+            kwargs['default'] = SUPPRESS
+            super().__init__(**kwargs)
 
-    def __call__(self, parser, namespace, value, option_string=None):
+    def __call__(self, *args, **kwargs):
+        if len(args) == 0:
+            kwargs['_basetype'] = self._basetype
+            return _ActionConfigLoad(**kwargs)
+        parser, namespace, value = args[:3]
         cfg_file = self._load_config(value)
-        for key, val in vars(cfg_file).items():
-            dest = self.dest+'.'+key
-            action = _find_action(parser, dest)
-            if isinstance(action, _ActionConfigLoad):
-                with change_to_path_dir(cfg_file.__path__):
-                    action(parser, namespace, val)
-            else:
-                setattr(namespace, dest, val)
+        cfg_file = {self.dest+'.'+k: v for k, v in vars(cfg_file).items()}
+        with change_to_path_dir(cfg_file.get(self.dest+'.__path__')):
+            parser._apply_actions(cfg_file)
+        for key, val in cfg_file.items():
+            setattr(namespace, key, val)
 
     def _load_config(self, value):
         try:
@@ -179,6 +225,9 @@ class _ActionConfigLoad(Action):
     def _check_type(self, value, cfg=None):
         return self._load_config(value)
 
+    def _instantiate_classes(self, value):
+        return self.basetype(**value)
+
 
 class _ActionHelpClassPath(Action):
 
@@ -207,6 +256,113 @@ class _ActionHelpClassPath(Action):
         args[0].exit()
 
 
+class _ActionLink(Action):
+
+    def __init__(
+        self,
+        parser,
+        source: Union[str, Tuple[str, ...]],
+        target: str,
+        compute_fn: Callable = None,
+    ):
+        self.parser = parser
+
+        # Set and check compute function
+        self.compute_fn = compute_fn
+        if compute_fn is None and not isinstance(source, str):
+            raise ValueError('Multiple source keys requires a compute function.')
+
+        # Set and check source and target actions
+        exclude = (_ActionLink, _ActionConfigLoad, _ActionSubCommands, ActionConfigFile)
+        source = (source,) if isinstance(source, str) else source
+        self.source = [(s, _find_parent_actions(parser, s, exclude=exclude)) for s in source]
+        self.target = (target, _find_parent_action(parser, target, exclude=exclude))
+        for key, action in self.source + [self.target]:
+            if action is None:
+                raise ValueError('No action for key "'+key+'".')
+
+        from .typehints import ActionTypeHint
+        is_target_subclass = ActionTypeHint.is_subclass_typehint(self.target[1])
+        valid_target_subclass = is_target_subclass and target.startswith(self.target[1].dest+'.init_args.')
+        valid_target_leaf = self.target[1].dest == target and not is_target_subclass
+        if not (valid_target_leaf or valid_target_subclass):
+            raise ValueError('Target key "'+target+'" must be for a individual argument.')
+
+        # Replace target action with link action
+        if not is_target_subclass:
+            for key in self.target[1].option_strings:
+                parser._option_string_actions[key] = self
+            parser._actions[parser._actions.index(self.target[1])] = self
+            for group in parser._action_groups:
+                if self.target[1] in group._group_actions:
+                    group._group_actions.remove(self.target[1])
+
+        # Add link action to group to show in help
+        if not hasattr(parser, '_links_group'):
+            parser._links_group = parser.add_argument_group('Linked arguments')
+        parser._links_group._group_actions.append(self)
+
+        # Initialize link action
+        link_str = target+' <= '
+        if compute_fn is None:
+            link_str += source[0]
+        else:
+            link_str += getattr(compute_fn, '__name__', str(compute_fn))+'('+', '.join(source)+')'
+
+        if is_target_subclass:
+            type_attr = None
+            help_str = 'Use --'+self.target[1].dest+'.help CLASS_PATH for details.'
+        else:
+            type_attr = getattr(self.target[1], '_typehint', self.target[1].type)
+            help_str = self.target[1].help
+
+        super().__init__(
+            [link_str],
+            dest=target,
+            default=SUPPRESS,
+            metavar='',
+            type=type_attr,
+            help=help_str,
+        )
+
+    def __call__(self, *args, **kwargs):
+        source = ', '.join(s[0] for s in self.source)
+        raise TypeError('Linked "'+self.target[0]+'" must be given via "'+source+'".')
+
+    def _check_type(self, value, cfg=None):
+        return self.parser._check_value_key(self.target[1], value, self.target[0], cfg)
+
+    @staticmethod
+    def propagate_arguments(parser, cfg_ns):
+        if hasattr(parser, '_links_group'):
+            for action in parser._links_group._group_actions:
+                try:
+                    args = []
+                    for key, _ in action.source:
+                        arg = _get_key_value(cfg_ns, key)
+                        if isinstance(arg, Namespace):
+                            arg = namespace_to_dict(arg)
+                        args.append(arg)
+                except AttributeError:
+                    continue
+                if action.compute_fn is None:
+                    value = args[0]
+                else:
+                    value = action.compute_fn(*args)
+                key = action.target[0]
+                parent_ns = cfg_ns
+                while True:
+                    if '.' not in key:
+                        setattr(parent_ns, key, value)
+                        break
+                    parent_key, key = key.rsplit('.', 1)
+                    try:
+                        parent_ns = _get_key_value(cfg_ns, parent_key)
+                    except AttributeError:
+                        value = dict_to_namespace({key: value})
+                        key = parent_key
+
+
 class ActionYesNo(Action):
     """Paired options --[yes_prefix]opt, --[no_prefix]opt to set True or False respectively."""
 
@@ -286,88 +442,6 @@ class ActionYesNo(Action):
         return ['true', 'false', 'yes', 'no']
 
 
-class ActionEnum(Action):
-    """An action based on an Enum that maps to-from strings and enum values."""
-
-    def __init__(
-        self,
-        enum: Enum = None,
-        **kwargs
-    ):
-        """Initializer for ActionEnum instance.
-
-        Args:
-            enum: An Enum class.
-
-        Raises:
-            ValueError: If a parameter is invalid.
-        """
-        if enum is not None:
-            if not _issubclass(enum, Enum):
-                raise ValueError('Expected enum to be an subclass of Enum.')
-            self._enum = enum
-        elif '_enum' not in kwargs:
-            raise ValueError('Expected enum keyword argument.')
-        else:
-            self._enum = kwargs.pop('_enum')
-            kwargs['metavar'] = '{'+','.join(self._enum.__members__.keys())+'}'  # type: ignore
-            super().__init__(**kwargs)
-
-    def __call__(self, *args, **kwargs):
-        """Parses an argument mapping a string to its Enum value.
-
-        Raises:
-            TypeError: If value not present in the Enum.
-        """
-        if len(args) == 0:
-            kwargs['_enum'] = self._enum
-            return ActionEnum(**kwargs)
-        setattr(args[1], self.dest, self._check_type(args[2]))
-
-    def _check_type(self, value, cfg=None):
-        islist = _is_action_value_list(self)
-        if not islist:
-            value = [value]
-        for num, val in enumerate(value):
-            try:
-                if isinstance(val, str):
-                    value[num] = self._enum[val]
-                else:
-                    self._enum(val)
-            except KeyError:
-                elem = '' if not islist else ' element '+str(num+1)
-                raise TypeError('Parser key "'+self.dest+'"'+elem+': value '+str(val)+' not in '+self._enum.__name__+'.')
-        return value if islist else value[0]
-
-    def completer(self, **kwargs):
-        """Used by argcomplete to support tab completion of arguments."""
-        return list(self._enum.__members__.keys())
-
-
-class ActionOperators:
-    """DEPRECATED: Action to restrict a value with comparison operators.
-
-    The new alternative is explained in :ref:`restricted-numbers`.
-    """
-
-    def __init__(self, **kwargs):
-        if 'expr' in kwargs:
-            restrictions = [kwargs['expr']] if isinstance(kwargs['expr'], tuple) else kwargs['expr']
-            register_key = (tuple(sorted(restrictions)), kwargs.get('type', int), kwargs.get('join', 'and'))
-            if register_key in registered_types:
-                self._type = registered_types[register_key]
-            else:
-                self._type = restricted_number_type(None, kwargs.get('type', int), kwargs['expr'], kwargs.get('join', 'and'))
-        else:
-            raise ValueError('Expected expr keyword argument.')
-
-    def __call__(self, *args, **kwargs):
-        if 'type' in kwargs:
-            raise ValueError('ActionOperators does not allow type given to add_argument.')
-        kwargs['type'] = self._type
-        return _StoreAction(**kwargs)
-
-
 class ActionParser:
     """Action to parse option with a given parser optionally loading from file if string value."""
 
