@@ -7,38 +7,38 @@ import yaml
 import logging
 import inspect
 import argparse
-from enum import Enum
 from copy import deepcopy
 from contextlib import redirect_stderr
-from typing import Any, List, Dict, Set, Union, Optional, Type, Callable
+from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Type, Union
 from argparse import ArgumentError, Action, Namespace, SUPPRESS
 
 from .formatters import DefaultHelpFormatter
-from .signatures import SignatureArguments
-from .typing import type_in
-from .jsonschema import ActionJsonSchema, supported_types
 from .jsonnet import ActionJsonnet
-from .optionals import (
-    dump_preserve_order_support,
-    import_jsonnet,
-    argcomplete_support,
-    import_argcomplete,
-    get_config_read_mode,
-    FilesCompleterMethod,
-    TypeCastCompleterMethod,
-)
+from .jsonschema import ActionJsonSchema
+from .signatures import is_pure_dataclass, SignatureArguments
+from .typehints import ActionTypeHint
 from .actions import (
-    ActionEnum,
     ActionParser,
     ActionConfigFile,
     ActionPath,
     _ActionSubCommands,
     _ActionPrintConfig,
     _ActionConfigLoad,
+    _ActionLink,
     _find_action,
+    _find_parent_action,
     _is_action_value_list,
     filter_default_actions,
 )
+from .optionals import (
+    argcomplete_support,
+    dump_preserve_order_support,
+    FilesCompleterMethod,
+    get_config_read_mode,
+    import_jsonnet,
+    import_argcomplete,
+    TypeCastCompleterMethod,
+)
 from .util import (
     namespace_to_dict,
     dict_to_namespace,
@@ -53,9 +53,10 @@ from .util import (
     change_to_path_dir,
     Path,
     LoggerProperty,
+    get_key_value_from_flat_dict,
+    update_key_value_in_flat_dict,
     _get_key_value,
     _get_env_var,
-    _issubclass,
     _suppress_stderr,
 )
 
@@ -75,7 +76,7 @@ default_dump_json_kwargs = {
 }
 
 
-class _ActionsContainer(argparse._ActionsContainer):
+class _ActionsContainer(SignatureArguments, argparse._ActionsContainer):
     """Extension of argparse._ActionsContainer to support additional functionalities."""
 
     def __init__(self, *args, **kwargs):
@@ -99,15 +100,15 @@ class _ActionsContainer(argparse._ActionsContainer):
                 raise ValueError('Parser cannot be added as a subparser of itself.')
             return ActionParser._move_parser_actions(parser, args, kwargs)
         if 'type' in kwargs:
-            if type_in(kwargs['type'], supported_types) or \
-               (inspect.isclass(kwargs['type']) and not _issubclass(kwargs['type'], (str, int, float, Enum, Path))):
+            if is_pure_dataclass(kwargs['type']):
+                theclass = kwargs.pop('type')
+                nested_key = re.sub('^--', '', args[0])
+                super().add_dataclass_arguments(theclass, nested_key, **kwargs)
+                return _find_action(parser, nested_key)
+            if ActionTypeHint.is_supported_typehint(kwargs['type']):
                 if 'action' in kwargs:
-                    raise ValueError('Type hints as type does not allow providing an action')
-                kwargs['action'] = ActionJsonSchema(annotation=kwargs.pop('type'), enable_path=enable_path)
-            elif _issubclass(kwargs['type'], Enum):
-                if 'action' in kwargs:
-                    raise ValueError('Enum as type does not allow providing an action')
-                kwargs['action'] = ActionEnum(enum=kwargs['type'])
+                    raise ValueError('Type hint as type does not allow providing an action')
+                kwargs['action'] = ActionTypeHint(typehint=kwargs.pop('type'), enable_path=enable_path)
         action = super().add_argument(*args, **kwargs)
         if not hasattr(action, 'completer') and action.type is not None:
             completer_method = FilesCompleterMethod if isinstance(action.type, Path) else TypeCastCompleterMethod
@@ -130,12 +131,39 @@ class _ActionsContainer(argparse._ActionsContainer):
         return action
 
 
+    def add_argument_group(self, *args, name:str=None, **kwargs):
+        """Adds a group to the parser.
+
+        All the arguments from `argparse.ArgumentParser.add_argument_group
+        <https://docs.python.org/3/library/argparse.html#argparse.ArgumentParser.add_argument_group>`_
+        are supported. Additionally it accepts:
+
+        Args:
+            name: Name of the group. If set the group object will be included in the parser.groups dict.
+
+        Returns:
+            The group object.
+
+        Raises:
+            ValueError: If group with the same name already exists.
+        """
+        parser = self.parser if hasattr(self, 'parser') else self  # type: ignore
+        if name is not None and name in parser.groups:
+            raise ValueError('Group with name '+name+' already exists.')
+        group = _ArgumentGroup(parser, *args, **kwargs)
+        group.parser = parser
+        parser._action_groups.append(group)
+        if name is not None:
+            parser.groups[name] = group
+        return group
+
+
 class _ArgumentGroup(_ActionsContainer, argparse._ArgumentGroup):
     """Extension of argparse._ArgumentGroup to support additional functionalities."""
     parser = None  # type: Union[ArgumentParser, None]
 
 
-class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentParser, LoggerProperty):
+class ArgumentParser(_ActionsContainer, argparse.ArgumentParser, LoggerProperty):
     """Parser for command line, yaml/jsonnet files and environment variables."""
 
     groups = None  # type: Dict[str, argparse._ArgumentGroup]
@@ -184,6 +212,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
         if self.groups is None:
             self.groups = {}
         self.required_args = set()  # type: Set[str]
+        self.save_path_content = set()  # type: Set[str]
         self._stderr = sys.stderr
         self._parse_as_dict = parse_as_dict
         self.default_config_files = default_config_files
@@ -225,9 +254,8 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
 
         try:
             namespace, args = self._parse_known_args(args, namespace)
-        except (ArgumentError, ParserError):
-            err = sys.exc_info()[1]
-            self.error(str(err))
+        except (ArgumentError, ParserError) as ex:
+            self.error(str(ex), ex)
 
         return namespace, args
 
@@ -240,6 +268,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
         nested: bool,
         with_meta: Optional[bool],
         skip_check: bool,
+        skip_required: bool = False,
         skip_subcommands: bool = False,
         fail_no_subcommand: bool = True,
         cfg_base: Union[Namespace, Dict[str, Any]] = None,
@@ -254,11 +283,14 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             nested: Whether the namespace should be nested.
             with_meta: Whether to include metadata in config object, None to use parser's default.
             skip_check: Whether to skip check if configuration is valid.
+            skip_required: Whether to skip check of required arguments.
+            skip_subcommands: Whether to skip subcommand processing.
+            fail_no_subcommand: Whether to fail if no subcommand given.
             cfg_base: A base configuration object.
             log_message: Message to log at INFO level after parsing.
 
         Returns:
-            A dict object with all parsed values.
+            A config object with all parsed values.
         """
         if env is None and self._default_env:
             env = True
@@ -286,12 +318,15 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
 
         cfg_ns = dict_to_namespace(cfg)
 
-        if hasattr(self, '_print_config'):
-            sys.stdout.write(self.dump(cfg_ns, skip_check=True, **self._print_config))  # type: ignore
-            self.exit()
+        _ActionPrintConfig.print_config_if_requested(self, cfg_ns)
+
+        _ActionLink.propagate_arguments(self, cfg_ns)
 
         if not skip_check:
-            self.check_config(cfg_ns)
+            self.check_config(cfg_ns, skip_required=skip_required)
+
+        if not nested:
+            cfg_ns = _dict_to_flat_namespace(namespace_to_dict(cfg_ns))
 
         if log_message is not None:
             self._logger.info(log_message)
@@ -323,7 +358,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             with_meta: Whether to include metadata in config object, None to use parser's default.
 
         Returns:
-            An object with all parsed values as nested attributes.
+            A config object with all parsed values.
 
         Raises:
             ParserError: If there is a parsing error and error_handler=None.
@@ -350,7 +385,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             )
 
         except (TypeError, KeyError) as ex:
-            self.error(str(ex))
+            self.error(str(ex), ex)
 
         return parsed_cfg
 
@@ -364,6 +399,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
         nested: bool = True,
         with_meta: bool = None,
         _skip_check: bool = False,
+        _skip_required: bool = False,
     ) -> Union[Namespace, Dict[str, Any]]:
         """Parses configuration given as an object.
 
@@ -375,14 +411,15 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             with_meta: Whether to include metadata in config object, None to use parser's default.
 
         Returns:
-            An object with all parsed values as attributes.
+            A config object with all parsed values.
 
         Raises:
             ParserError: If there is a parsing error and error_handler=None.
         """
         try:
             cfg = vars(_dict_to_flat_namespace(cfg_obj))
-            self._apply_actions(cfg, self._actions)
+            self._apply_actions(cfg)
+            cfg = _flat_namespace_to_dict(dict_to_namespace(cfg))
 
             parsed_cfg = self._parse_common(
                 cfg=cfg,
@@ -391,12 +428,13 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
                 nested=nested,
                 with_meta=with_meta,
                 skip_check=_skip_check,
+                skip_required=_skip_required,
                 cfg_base=cfg_base,
                 log_message='Parsed object.',
             )
 
         except (TypeError, KeyError) as ex:
-            self.error(str(ex))
+            self.error(str(ex), ex)
 
         return parsed_cfg
 
@@ -420,7 +458,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             with_meta: Whether to include metadata in config object, None to use parser's default.
 
         Returns:
-            An object with all parsed values as attributes.
+            A config object with all parsed values.
 
         Raises:
             ParserError: If there is a parsing error and error_handler=None.
@@ -471,7 +509,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             )
 
         except (TypeError, KeyError) as ex:
-            self.error(str(ex))
+            self.error(str(ex), ex)
 
         return parsed_cfg
 
@@ -499,7 +537,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             with_meta: Whether to include metadata in config object, None to use parser's default.
 
         Returns:
-            An object with all parsed values as nested attributes.
+            A config object with all parsed values.
 
         Raises:
             ParserError: If there is a parsing error and error_handler=None.
@@ -548,7 +586,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             with_meta: Whether to include metadata in config object, None to use parser's default.
 
         Returns:
-            An object with all parsed values as attributes.
+            A config object with all parsed values.
 
         Raises:
             ParserError: If there is a parsing error and error_handler=None.
@@ -568,7 +606,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             )
 
         except (TypeError, KeyError) as ex:
-            self.error(str(ex))
+            self.error(str(ex), ex)
 
         return parsed_cfg
 
@@ -600,39 +638,38 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             raise TypeError('Problems parsing config :: '+str(ex)) from ex
         cfg = namespace_to_dict(_dict_to_flat_namespace(cfg))
 
-        self._apply_actions(cfg, self._actions)
+        self._apply_actions(cfg)
 
         return cfg
 
 
-    ## Methods for adding to the parser ##
+    def link_arguments(
+        self,
+        source: Union[str, Tuple[str, ...]],
+        target: str,
+        compute_fn: Callable = None,
+    ):
+        """Makes an argument value be derived from the values other arguments.
 
-    def add_argument_group(self, *args, name:str=None, **kwargs):
-        """Adds a group to the parser.
-
-        All the arguments from `argparse.ArgumentParser.add_argument_group
-        <https://docs.python.org/3/library/argparse.html#argparse.ArgumentParser.add_argument_group>`_
-        are supported. Additionally it accepts:
+        Source keys can be individual arguments or nested groups. The target key
+        has to be an single argument. The keys can be inside init_args of a
+        subclass. The compute function should accept as many positional
+        arguments as there are sources and return a value of type compatible
+        with the target.
 
         Args:
-            name: Name of the group. If set the group object will be included in the parser.groups dict.
-
-        Returns:
-            The group object.
+            source: Key(s) from which the target value is derived.
+            target: Key to where the value is set.
+            compute_fn: Function to compute target value from source.
 
         Raises:
-            ValueError: If group with the same name already exists.
+            ValueError: If an invalid parameter is given.
         """
-        if name is not None and name in self.groups:
-            raise ValueError('Group with name '+name+' already exists.')
-        group = _ArgumentGroup(self, *args, **kwargs)
-        group.parser = self
-        self._action_groups.append(group)
-        if name is not None:
-            self.groups[name] = group
-        return group
+        _ActionLink(self, source, target, compute_fn)
 
 
+    ## Methods for adding to the parser ##
+
     def add_subparsers(self, **kwargs):
         """Raises a NotImplementedError since jsonargparse uses add_subcommands."""
         raise NotImplementedError('In jsonargparse sub-commands are added using the add_subcommands method.')
@@ -696,43 +733,31 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             self.check_config(cfg)
 
         def cleanup_actions(cfg, actions):
-            for action in actions:
+            for action in filter_default_actions(actions):
                 if (action.help == SUPPRESS and not isinstance(action, _ActionConfigLoad)) or \
                    isinstance(action, ActionConfigFile) or \
                    (skip_none and action.dest in cfg and cfg[action.dest] is None):
-                    del cfg[action.dest]
+                    cfg.pop(action.dest, None)
                 elif isinstance(action, ActionPath):
                     if cfg[action.dest] is not None:
                         if isinstance(cfg[action.dest], list):
                             cfg[action.dest] = [str(p) for p in cfg[action.dest]]
                         else:
                             cfg[action.dest] = str(cfg[action.dest])
-                elif isinstance(action, ActionJsonSchema) and action._annotation is not None and cfg.get(action.dest) is not None:
-                    cfg[action.dest] = ActionJsonSchema._adapt_types(cfg[action.dest], action._annotation, action._subschemas, True)
-
-        def cleanup_types(cfg):
-            for dest, val in cfg.items():
-                if isinstance(val, Enum):
-                    cfg[dest] = val.name
-                elif isinstance(val, Path):
-                    cfg[dest] = str(val)
-                elif 'jsonargparse.typing.' in str(type(val)):
-                    if isinstance(val, str):
-                        cfg[dest] = str(val)
-                    elif isinstance(val, int):
-                        cfg[dest] = int(val)
-                    elif isinstance(val, float):
-                        cfg[dest] = float(val)
+                elif isinstance(action, ActionTypeHint):
+                    value = get_key_value_from_flat_dict(cfg, action.dest)
+                    if value is not None and value != {}:
+                        value = ActionTypeHint.serialize(value, action._typehint)
+                        update_key_value_in_flat_dict(cfg, action.dest, value)
 
         cfg = namespace_to_dict(_dict_to_flat_namespace(cfg))
         cleanup_actions(cfg, self._actions)
-        cleanup_types(cfg)
         cfg = _flat_namespace_to_dict(_dict_to_flat_namespace(cfg))
 
         if format == 'parser_mode':
             format = 'yaml' if self.parser_mode == 'yaml' else 'json_indented'
         if format == 'yaml':
-            return yaml.dump(cfg, **self.dump_yaml_kwargs)  # type: ignore
+            return yaml.safe_dump(cfg, **self.dump_yaml_kwargs)  # type: ignore
         elif format == 'json_indented':
             return json.dumps(cfg, indent=2, **self.dump_json_kwargs)+'\n'  # type: ignore
         elif format == 'json':
@@ -766,14 +791,17 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
         Raises:
             TypeError: If any of the values of cfg is invalid according to the parser.
         """
-        if not overwrite and os.path.isfile(path):
-            raise ValueError('Refusing to overwrite existing file: '+path)
-        path = Path(path, mode='fc')
+        def check_overwrite(path):
+            if not overwrite and os.path.isfile(path()):
+                raise ValueError('Refusing to overwrite existing file: '+path())
+
+        path_fc = Path(path, mode='fc')
+        check_overwrite(path_fc)
 
         dump_kwargs = {'format': format, 'skip_none': skip_none, 'skip_check': skip_check}
 
         if not multifile:
-            with open(path(), 'w') as f:
+            with open(path_fc(), 'w') as f:
                 f.write(self.dump(cfg, **dump_kwargs))  # type: ignore
 
         else:
@@ -784,39 +812,42 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             if not skip_check:
                 self.check_config(strip_meta(cfg), branch=branch)
 
-            dirname = os.path.dirname(path())
-            save_kwargs = deepcopy(dump_kwargs)
-            save_kwargs.update({'overwrite': overwrite, 'multifile': multifile})
-
             def save_paths(cfg, base=None):
                 replace_keys = {}
                 for key, val in cfg.items():
+                    full_key = ('' if base is None else base+'.')+key
                     if isinstance(val, dict):
                         kbase = str(key) if base is None else base+'.'+str(key)
                         if '__path__' in val:
-                            val_path = Path(os.path.join(dirname, os.path.basename(val['__path__']())), mode='fc')
-                            if not overwrite and os.path.isfile(val_path()):
-                                raise ValueError('Refusing to overwrite existing file: '+str(val_path))
+                            val_path = Path(os.path.basename(val['__path__']()), mode='fc')
+                            check_overwrite(val_path)
                             action = _find_action(self, kbase)
                             if isinstance(action, (ActionJsonSchema, ActionJsonnet, _ActionConfigLoad)):
-                                replace_keys[key] = val_path
                                 val_out = strip_meta(val)
                                 if '__orig__' in val:
                                     val_str = val['__orig__']
                                 elif str(val_path).lower().endswith('.json'):
                                     val_str = json.dumps(val_out, indent=2, **self.dump_json_kwargs)+'\n'
                                 else:
-                                    val_str = yaml.dump(val_out, **self.dump_yaml_kwargs)
+                                    val_str = yaml.safe_dump(val_out, **self.dump_yaml_kwargs)
                                 with open(val_path(), 'w') as f:
                                     f.write(val_str)
+                                replace_keys[key] = os.path.basename(val_path())
                         else:
                             save_paths(val, kbase)
+                    elif isinstance(val, Path) and full_key in self.save_path_content and 'r' in val.mode:
+                        val_path = Path(os.path.basename(val()), mode='fc')
+                        check_overwrite(val_path)
+                        with open(val_path(), 'w') as f:
+                            f.write(val.get_content())
+                        replace_keys[key] = type(val)(str(val_path))
                 for key, val in replace_keys.items():
-                    cfg[key] = os.path.basename(val())
+                    cfg[key] = val
 
-            save_paths(cfg)
+            with change_to_path_dir(path_fc):
+                save_paths(cfg)
             dump_kwargs['skip_check'] = True
-            with open(path(), 'w') as f:
+            with open(path_fc(), 'w') as f:
                 f.write(self.dump(cfg, **dump_kwargs))  # type: ignore
 
 
@@ -836,7 +867,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             for n in range(len(args)):
                 self._defaults.update(args[n])
                 for dest in args[n].keys():
-                    action = _find_action(self, dest)
+                    action = _find_action(self, dest, within_subcommands=True)
                     if action is None:
                         raise KeyError('No action for destination key "'+dest+'" to set its default.')
                     action.default = args[n][dest]
@@ -884,7 +915,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             An object with all default values as attributes.
         """
         cfg = {}
-        for action in self._actions:
+        for action in filter_default_actions(self._actions):
             if action.default != SUPPRESS and action.dest != SUPPRESS:
                 cfg[action.dest] = action.default
 
@@ -896,34 +927,45 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
         if default_config_file is not None:
             with change_to_path_dir(default_config_file):
                 cfg_file = self._load_cfg(default_config_file.get_content())
-                if not skip_check:
-                    try:
-                        self.check_config(cfg_file, skip_required=True)
-                    except (TypeError, KeyError) as ex:
-                        raise ParserError('Problem in default config file "'+str(default_config_file)+'" :: '+ex.args[0]) from ex
+                try:
+                    self.print_config_skip = True
+                    cfg_file = self.parse_object(  # type: ignore
+                        _flat_namespace_to_dict(dict_to_namespace(cfg_file)),
+                        defaults=False,
+                        nested=False,
+                        _skip_check=skip_check,
+                        _skip_required=True,
+                    )
+                    delattr(self, 'print_config_skip')
+                except (TypeError, KeyError, ParserError) as ex:
+                    raise ParserError('Problem in default config file "'+str(default_config_file)+'" :: '+ex.args[0]) from ex
             cfg = self._merge_config(cfg_file, cfg)
             cfg['__default_config__'] = default_config_file
             self._logger.info('Parsed configuration from default path: %s', str(default_config_file))
 
         if nested:
-            cfg = _flat_namespace_to_dict(Namespace(**cfg))
+            cfg = _flat_namespace_to_dict(dict_to_namespace(cfg))
 
         return dict_to_namespace(cfg)
 
 
     ## Other methods ##
 
-    def error(self, message:str):
+    def error(self, message:str, ex:Exception=None):
         """Logs error message if a logger is set, calls the error handler and raises a ParserError."""
         self._logger.error(message)
         if self._error_handler is not None:
             with redirect_stderr(self._stderr):
                 self._error_handler(self, message)
-        raise ParserError(message)
+        if ex is None:
+            raise ParserError(message)
+        else:
+            raise ParserError(message) from ex
 
 
     def check_config(
-        self, cfg: Union[Namespace, Dict[str, Any]],
+        self,
+        cfg: Union[Namespace, Dict[str, Any]],
         skip_none: bool = True,
         skip_required: bool = False,
         branch: str = None,
@@ -957,8 +999,8 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
                     val = get_key_value(cfg, reqkey)
                     if val is None:
                         raise TypeError()
-                except (KeyError, TypeError):
-                    raise TypeError('Key "'+reqkey+'" is required but not included in config object or its value is None.')
+                except (KeyError, TypeError) as ex:
+                    raise TypeError('Key "'+reqkey+'" is required but not included in config object or its value is None.') from ex
 
         def check_values(cfg, base=None):
             subcommand = None
@@ -975,6 +1017,8 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
                         if subcommand is not None:
                             raise KeyError('Only values from a single sub-command are allowed ("'+subcommand+'", "'+kbase+'").')
                         subcommand = kbase
+                    elif isinstance(action, _ActionConfigLoad) and isinstance(val, dict):
+                        check_values(val, kbase)
                 elif isinstance(val, dict):
                     check_values(val, kbase)
                 else:
@@ -992,7 +1036,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             raise type(ex)(message) from ex
 
 
-    def instantiate_subclasses(self, cfg:Union[Namespace, Dict[str, Any]]) -> Union[Namespace, Dict[str, Any]]:
+    def instantiate_subclasses(self, cfg:Union[Namespace, Dict[str, Any]]) -> Dict[str, Any]:
         """Recursively instantiates all subclasses defined by 'class_path' and 'init_args'.
 
         Args:
@@ -1001,16 +1045,23 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
         Returns:
             A configuration object with all subclasses instantiated.
         """
-        cfg = namespace_to_dict(strip_meta(cfg))
-        for action in self._actions:
-            if isinstance(action, ActionJsonSchema):
-                try:
-                    val, par, key = _get_key_value(cfg, action.dest, parent=True)
-                except KeyError:
-                    pass
-                else:
-                    par[key] = action._instantiate_classes(val)
-        return cfg if self._parse_as_dict else dict_to_namespace(cfg)
+        cfg = strip_meta(cfg)
+        actions = filter_default_actions(self._actions)
+        actions.sort(key=lambda x: -len(x.dest.split('.')))
+        for action in actions:
+            if isinstance(action, ActionTypeHint) or \
+               (isinstance(action, _ActionConfigLoad) and is_pure_dataclass(action.basetype)):
+                value, parent, key = _get_key_value(cfg, action.dest, parent=True)
+                if value is not None:
+                    parent[key] = action._instantiate_classes(value)
+                #try:
+                #    value, parent, key = _get_key_value(cfg, action.dest, parent=True)
+                #except KeyError:
+                #    pass
+                #else:
+                #    if value is not None:
+                #        parent[key] = action._instantiate_classes(value)
+        return cfg
 
 
     def strip_unknown(self, cfg:Union[Namespace, Dict[str, Any]]) -> Namespace:
@@ -1056,7 +1107,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
         cfg_files = []
         if '__default_config__' in cfg:
             cfg_files.append(cfg['__default_config__'])
-        for action in self._actions:
+        for action in filter_default_actions(self._actions):
             if isinstance(action, ActionConfigFile) and action.dest in cfg and cfg[action.dest] is not None:
                 cfg_files.extend(p for p in cfg[action.dest] if p is not None)
         return cfg_files
@@ -1068,7 +1119,7 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
             note = 'no existing default config file found.'
             if '__default_config__' in defaults:
                 note = 'default values below will be ones overridden by the contents of: '+str(defaults['__default_config__'])
-                self.formatter_class.defaults = vars(_dict_to_flat_namespace(defaults))
+                self.formatter_class.defaults = defaults
             group = self._default_config_files_group
             group.description = str(self._default_config_files) + ', Note: '+note
         help_str = super().format_help()
@@ -1077,12 +1128,24 @@ class ArgumentParser(SignatureArguments, _ActionsContainer, argparse.ArgumentPar
         return help_str
 
 
-    def _apply_actions(self, cfg, actions):
+    def _apply_actions(self, cfg):
         """Runs _check_value_key on actions present in flat config dict."""
-        for action in actions:
-            if action.dest in cfg:
-                value = self._check_value_key(action, cfg[action.dest], action.dest, cfg)
-                cfg[action.dest] = value
+        keys = [k for k in cfg.keys() if k.rsplit('.', 1)[-1] not in meta_keys]
+        keys.sort(key=lambda x: -len(x.split('.')))
+        seen_keys = set()
+        for key in keys:
+            if key in seen_keys:
+                continue
+            action = _find_parent_action(self, key)
+            if action is not None:
+                if action.dest == key:
+                    value = self._check_value_key(action, cfg[action.dest], action.dest, cfg)
+                    cfg[action.dest] = value
+                else:
+                    value = get_key_value_from_flat_dict(cfg, action.dest)
+                    value = self._check_value_key(action, value, action.dest, cfg)
+                    update_key_value_in_flat_dict(cfg, action.dest, value)
+                    seen_keys.update(action.dest+'.'+k for k in value.keys())
 
 
     @staticmethod
