@@ -293,6 +293,20 @@ class ArgumentFeaturesTests(unittest.TestCase):
 
 class AdvancedFeaturesTests(unittest.TestCase):
 
+    def test_link_arguments(self):
+        parser = ArgumentParser(error_handler=None)
+        parser.add_argument('--a.v1', default=2)
+        parser.add_argument('--a.v2', type=int, default=3)
+        parser.add_argument('--b.v2', type=int, default=4)
+        def a_prod(a):
+            return a['v1'] * a['v2']
+        parser.link_arguments('a', 'b.v2', a_prod)
+
+        cfg = parser.parse_args(['--a.v2=-5'])
+        self.assertEqual(cfg.b.v2, cfg.a.v1*cfg.a.v2)
+        self.assertRaises(ParserError, lambda: parser.parse_args(['--a.v1=x']))
+
+
     def test_subcommands(self):
         parser_a = ArgumentParser(error_handler=None)
         parser_a.add_argument('ap1')
@@ -641,6 +655,31 @@ class OutputTests(TempDirTestCase):
             self.assertTrue(os.path.isfile(os.path.join(outdir, 'schema.yaml')))
 
 
+    def test_save_path_content(self):
+        parser = ArgumentParser()
+        parser.add_argument('--the.path', type=Path_fr)
+
+        os.mkdir('pathdir')
+        os.mkdir('outdir')
+        file_txt = os.path.join('pathdir', 'file.txt')
+        out_yaml = os.path.join('outdir', 'saved.yaml')
+        out_file = os.path.join('outdir', 'file.txt')
+
+        with open(file_txt, 'w') as output_file:
+            output_file.write('file content')
+
+        cfg = parser.parse_args(['--the.path', file_txt])
+        parser.save_path_content.add('the.path')
+        parser.save(cfg, out_yaml)
+
+        self.assertTrue(os.path.isfile(out_yaml))
+        self.assertTrue(os.path.isfile(out_file))
+        with open(out_yaml) as input_file:
+            self.assertEqual(input_file.read(), 'the:\n  path: file.txt\n')
+        with open(out_file) as input_file:
+            self.assertEqual(input_file.read(), 'file content')
+
+
     def test_save_failures(self):
         parser = ArgumentParser()
         with open('existing.yaml', 'w') as output_file:
@@ -706,7 +745,7 @@ class ConfigFilesTests(TempDirTestCase):
         with open(default_config_file, 'w') as output_file:
             output_file.write('op1: from yaml\n')
 
-        parser = ArgumentParser(default_config_files=[default_config_file])
+        parser = ArgumentParser(default_config_files=[default_config_file], error_handler=None)
         parser.add_argument('--op1', default='from default')
 
         self.assertEqual(parser.get_default('op1'), 'from yaml')
